@@ -20,6 +20,9 @@ class mOTU:
         return "< {tax} mOTU {name}, of {len} members >".format(name = self.name, len = len(self), tax =  None ) #self.consensus_tax()[0].split(";")[-1])
 
     def __init__(self, **kwargs):
+        self.quiet = kwargs['quiet'] if "quiet" in kwargs else False
+        self.likelies = None
+        self.mock = []
         if "cog_dict" in kwargs:
             self.__for_mOTUpan(**kwargs)
 
@@ -27,12 +30,13 @@ class mOTU:
             self.__from_bins(**kwargs)
 
 
-    def __for_mOTUpan(self, name, faas, cog_dict, checkm_dict, max_it = 20):
+    def __for_mOTUpan(self, name, faas, cog_dict, checkm_dict, threads = 4, precluster = False, max_it = 20, method = None, quiet=False):
         self.name = name
         self.faas = faas
-        print("Creating mOTU for mOTUpan")
+        if not quiet:
+            print("Creating mOTU for mOTUpan", file = sys.stderr)
         if  not cog_dict :
-            tt = compute_COGs(self.faas, name = name + "COG")
+            tt = compute_COGs(self.faas, name = name, precluster = precluster, threads = threads)
             self.cog_dict = tt['genome2cogs']
             self.aa2cog = tt['aa2cog']
         else :
@@ -46,7 +50,6 @@ class mOTU:
                 checkm_dict[f] = 100*len(self.cog_dict[f])/max_len
 
         self.members = [MetaBin(bin_name, cogs = self.cog_dict[bin_name], faas = self.faas.get(bin_name), fnas = None, complet = checkm_dict.get(bin_name)) for bin_name in self.cog_dict.keys()]
-        self.mock = None
         self.cogCounts = {c : 0 for c in set.union(set([cog for mag in self.members for cog in mag.cogs]))}
         for mag in self.members:
             for cog in mag.cogs:
@@ -54,26 +57,47 @@ class mOTU:
 
         self.core = {cog for cog, counts in self.cogCounts.items() if (100*counts/len(self)) > mean(checkm_dict.values())}
 
-        self.likelies = self.__core_likelyhood(max_it = max_it)
+        self.method = method
+        if self.method :
+            self.likelies = self.__core_likelyhood(max_it = max_it)
+
 
     def __getitem__(self, i):
         return self.members[i]
 
 
-    def roc_values(self):
-        from mOTUlizer.classes.MockData import MockmOTU
+    def roc_values(self, boots):
+        if boots > 0:
+            from mOTUlizer.classes.MockData import MockmOTU
+            mean = lambda data: float(sum(data)/len(data))
+            variance 	= lambda data, avg: sum([x**2 for x in [i-avg for i in data]])/float(len(data))
+            std_dev = lambda data: variance(data, mean(data))**0.5
 
-        if self.mock is None:
-            completnesses = {"Genome_{}".format(i) : c.new_completness for i,c in enumerate(self)}
+            while len(self.mock) < boots:
+                print("Running bootstrap {}/{}".format(len(self.mock)+1, boots), file = sys.stderr)
+                completnesses = {"Genome_{}".format(i) : c.new_completness for i,c in enumerate(self)}
 
-            accessory = sorted([v for k,v in self.cogCounts.items() if k not in self.core])
-            missing = int(sum(accessory)*(1-mean(list(completnesses.values()))/100))
-            addeds = choices(list(range(len(accessory))), weights = accessory, k = missing)
-            for k in addeds :
-                accessory[k] += 1
+                accessory = sorted([v for k,v in self.cogCounts.items() if k not in self.core])
+                missing = int(sum(accessory)*(1-mean(list(completnesses.values()))/100))
+                if len(accessory) > 0:
+                    addeds = choices(list(range(len(accessory))), weights = accessory, k = missing)
+                    for k in addeds :
+                        accessory[k] += 1
 
-            self.mock = MockmOTU(self.name + "_mock", len(self.core), len(self), lambda g : completnesses[g], accessory = accessory)
-        return { 'recall' : self.mock.recall, 'lowest_false' : self.mock.lowest_false, 'fpr' : self.mock.fpr }
+                self.mock += [MockmOTU(self.name + "_mock", len(self.core), len(self), lambda g : completnesses[g], accessory = accessory, method = self.method)]
+            return { 'mean_recall' : mean([m.recall for m in self.mock]),
+                     'sd_recall' : std_dev([m.recall for m in self.mock]),
+                     'mean_fpr' : mean([m.fpr for m in self.mock]),
+                     'sd_fpr' : std_dev([m.fpr for m in self.mock]),
+                     'mean_lowest_false' : mean([m.lowest_false for m in self.mock]),
+                     'sd_lowest_false' : std_dev([m.lowest_false for m in self.mock])}
+        else:
+            return { 'mean_recall' : "NA",
+                     'sd_recall' : "NA",
+                     'mean_fpr' : "NA",
+                     'sd_fpr' : "NA",
+                     'mean_lowest_false' : "NA",
+                     'sd_lowest_false' : "NA"}
 
 
     def avg_cog_content(self):
@@ -106,6 +130,9 @@ class mOTU:
 
     def fastani_matrix(self):
         if not hasattr(self, 'fastani_dict'):
+            if not shutil.which('fastANI'):
+                print("You need fastANI if you do not provide a file with pairwise similarities, either install it or provide pairwise similarities (see doc...)", file = sys.stderr)
+                sys.exit(-1)
             cmd = "fastANI --ql {temp_file}  --rl {temp_file}  -o {temp_out} -t {threads} 2> /dev/null"
             temp_inp = tempfile.NamedTemporaryFile()
             temp_out = tempfile.NamedTemporaryFile()
@@ -130,6 +157,7 @@ class mOTU:
 
     def get_stats(self):
         out = {}
+
         out[self.name] = { "nb_genomes" : len(self),
         "core" : list(self.core) if self.core else None,
         "aux_genome" : [k for k,v in self.cogCounts.items() if k not in self.core] if self.core else None ,
@@ -142,6 +170,32 @@ class mOTU:
         }
         return out
 
+    def get_representative(self, method = "complex", max_redund = 5, min_complete = 95):
+            tt = [v.get_data() for v in self]
+            data = { t['name'] : t for t in tt}
+            if all([v['checkm_contamin'] > max_redund for v in data.values()]):
+                return max( [ (k , v['checkm_complet']) for k,v in data.items()], key = lambda x: x[1])[0]
+            data = {k : v for k,v in data.items() if v['checkm_contamin'] < max_redund}
+            if any([v['checkm_complet'] > min_complete for v in data.values()])  :
+                data = {k : v for k,v in data.items() if v['checkm_complet'] > min_complete}
+                best_redund = min(data.items(), key = lambda x : x[1]['checkm_contamin'])[1]['checkm_contamin']
+                return max( [ (k , v['checkm_complet']) for k,v in data.items() if v['checkm_contamin'] == best_redund], key = lambda x: x[1])[0]
+            else:
+                return max( [ (k , v['checkm_complet']) for k,v in data.items()], key = lambda x: x[1])[0]
+
+    # def get_representative(tt, max_redund = 5, min_complete = 95):
+    #         data = { t['name'] : t for t in tt}
+    #
+    #         data = {k : v for k,v in data.items() if v['checkm_contamin'] < max_redund}
+    #         if any([v['checkm_complet'] > min_complete for v in data.values()])  :
+    #             data = {k : v for k,v in data.items() if v['checkm_complet'] > min_complete}
+    #             best_redund = min(data.items(), key = lambda x : x[1]['checkm_contamin'])[1]['checkm_contamin']
+    #             return min( [ (k , v['checkm_contamin']) for k,v in data.items() if v['checkm_contamin'] == best_redund], key = lambda x: x[1])[0]
+    #         elif len(data) >0 :
+    #             return max( [ (k , v['checkm_complet']) for k,v in data.items()], key = lambda x: x[1])[0]
+    #         else :
+    #             return None
+
     def get_mean_ani(self):
         dist_dict = self.fastani_matrix()
         dists = [dist_dict.get((a.name,b.name)) for a in self for b in self if a != b ]
@@ -155,7 +209,8 @@ class mOTU:
         self.core = set([c for c, v in likelies.items() if v > likeli_cutof])
         core_len = len(self.core)
         i = 1
-        print("iteration 1 : ", core_len, "LHR:" , sum(likelies.values()), file = sys.stderr)
+        if not self.quiet:
+            print("iteration 1 : ", core_len, "sum_abs_LLHR:" , sum([l if l > 0 else -l for l in likelies.values()]), file = sys.stderr)
         for mag in self:
             if len(self.core) > 0:
                 mag.new_completness = 100*len(mag.cogs.intersection(self.core))/len(self.core)
@@ -175,14 +230,16 @@ class mOTU:
                     mag.new_completness = 0
                 mag.new_completness = mag.new_completness if mag.new_completness < 99.9 else 99.9
                 mag.new_completness = mag.new_completness if mag.new_completness > 0 else 0.01
-            print("iteration",i, ": ", new_core_len, "LHR:" , sum(likelies.values()), file = sys.stderr)
+            if not self.quiet:
+                print("iteration",i, ": ", new_core_len, "sum_abs_LLHR:" , sum([l if l > 0 else -l for l in likelies.values()]), file = sys.stderr)
             if self.core == old_core:
                break
             else :
-                core_len =new_core_len
+                core_len = new_core_len
 
-        json.dump({ self.name : {"nb_mags" : len(self), "core_len" : core_len, "mean_starting_completeness" :  mean([b.checkm_complet for b in self]), "mean_new_completness" : mean([b.new_completness for b in self]), "LHR"  :  sum([l if l > 0 else -l for l in likelies.values()]), "mean_est_binsize" : mean([100*len(b.cogs)/b.new_completness for b in self])}}, sys.stderr )
-        print(file = sys.stderr)
+        if not self.quiet:
+            json.dump({ self.name : {"nb_mags" : len(self), "core_len" : core_len, "mean_starting_completeness" :  mean([b.checkm_complet for b in self]), "mean_new_completness" : mean([b.new_completness for b in self]), "LHR"  :  sum([l if l > 0 else -l for l in likelies.values()]), "mean_est_binsize" : mean([100*len(b.cogs)/b.new_completness for b in self])}}, sys.stderr )
+            print(file = sys.stderr)
         self.iterations = i -1
         return likelies
 
@@ -239,7 +296,7 @@ class mOTU:
     def cluster_MetaBins(cls , all_bins, dist_dict, ani_cutoff = 95, prefix = "mOTU_", mag_complete = 40, mag_contamin = 5, sub_complete = 0, sub_contamin = 100):
         import igraph
 
-        print("seeding bin-graph")
+        print("seeding bin-graph", file = sys.stderr )
 
         all_bins = {a.name : a for a in all_bins}
 
@@ -254,22 +311,22 @@ class mOTU:
         species_graph.add_vertices(len(vertexDeict))
         species_graph.add_edges([(vertexDeict[k[0]], vertexDeict[k[1]]) for k in good_pairs])
 
-        print("getting clusters")
+        print("getting clusters", file = sys.stderr)
 
         genome_clusters = [[rev_vertexDeict[cc] for cc in c ] for c in species_graph.components(mode=igraph.STRONG)]
 
         mean = lambda l : sum([len(ll) for ll in l])/len(l)
 
-        print("recruiting to graph of the", len(genome_clusters) ," mOTUs of mean length", mean(genome_clusters))
+        print("recruiting to graph of the", len(genome_clusters) ," mOTUs of mean length", mean(genome_clusters), file = sys.stderr)
 
 
         left_pairs = {k : v for k, v in dist_dict.items() if v > ani_cutoff and k[0] != k[1] and ((decent_sub(k[0]) and good_mag(k[1])) or (decent_sub(k[1]) and good_mag(k[0])))}
-        print("looking for good_left pairs")
+        print("looking for good_left pairs", file = sys.stderr)
 #        print(left_pairs)
 
         subs = {l : (None,0) for ll in left_pairs.keys() for l in ll if not good_mag(l)}
 #        print(subs)
-        print("looking for best mOTU match")
+        print("looking for best mOTU match", file = sys.stderr)
         for p,ani in left_pairs.items():
             if p[0] in subs and subs[p[0]][1] < ani:
                 subs[p[0]] = (p[1], ani)
@@ -278,7 +335,7 @@ class mOTU:
 
         genome_clusters = [set(gg) for gg in genome_clusters]
 
-        print("append to the", len(genome_clusters) ,"mOTUs of mean length", mean(genome_clusters))
+        print("append to the", len(genome_clusters) ,"mOTUs of mean length", mean(genome_clusters), file = sys.stderr)
         for k, v in subs.items():
             for g in genome_clusters:
                 if v[0] in g :
@@ -286,7 +343,7 @@ class mOTU:
 
         genome_clusters = [list(gg) for gg in genome_clusters]
 
-        print("processing the", len(genome_clusters) ,"mOTUs of mean length", mean(genome_clusters))
+        print("processing the", len(genome_clusters) ,"mOTUs of mean length", mean(genome_clusters), file = sys.stderr)
         #print(genome_clusters)
 
         zeros = len(str(len(genome_clusters)))
