@@ -17,15 +17,14 @@ def parse_checkm(file):
     all_lines = [re.sub(r"  +","\t", a).split("\t") for a in all_lines]
 
     header_lines = [i for i,l in enumerate(all_lines) if 'Bin Id' in l and 'Completeness' in l and 'Contamination' in l]
-    assert len(header_lines) == 1, "your completness files is badly formed, it should be TAB-separated and needs a header line with 'Bin Id', 'Completeness', and 'Contamination' in it"
+    assert len(header_lines) >= 1, "your completness files is badly formed, it should be TAB-separated (multispaced...) and needs a header line with 'Bin Id', 'Completeness', and 'Contamination' in it"
 
     header_lines = header_lines[0]
     header_line = all_lines[header_lines]
 
-    lines = [l for i,l in enumerate(all_lines) if i != header_lines and len(l) == len(header_line)]
+    lines = [l for i,l in enumerate(all_lines) if i != header_lines and len(l) == len(header_line) and 'Completeness' not in l]
 
     assert len(lines) > 0, "there are no lines in the file, you sure the serparators are 'OK'"
 
-    lines = [{a : b if a in ['Marker lineage', 'Bin Id'] else float(b) for a,b in zip(header_line,l) }for l in lines]
-
+    lines = [{a : b if a not in ['Completeness', 'Contamination'] else float(b) for a,b in zip(header_line,l) }for l in lines]
     return {l['Bin Id'] : l for l in lines}
